@@ -41,6 +41,7 @@ from . import cache
 from . import compress
 from . import config
 from . import controller
+from . import crypt
 from . import data
 from . import defines
 from . import exceptions
@@ -71,11 +72,13 @@ from .api import Api, OAuthApi, OAuth1Api, OAuth2Api
 from .async import AsyncManager, SimpleManager, QueueManager
 from .base import APP, LEVEL, NAME, VERSION, PLATFORM, API_VERSION, BUFFER_SIZE, MAX_LOG_SIZE,\
     MAX_LOG_COUNT, App, APIApp, WebApp, get_app, get_name, get_base_path, get_request,\
-    get_session, get_model, get_controller, get_adapter, get_manager, get_logger, get_level, is_devel, is_safe
+    get_session, get_model, get_controller, get_adapter, get_manager, get_logger, get_level,\
+    is_devel, is_safe, to_locale, on_exit
 from .cache import Cache, MemoryCache
 from .compress import Compress
 from .config import conf, conf_prefix, conf_s
 from .controller import Controller
+from .crypt import Cipher, RC4, Spritz
 from .data import DataAdapter, MongoAdapter, TinyAdapter, Collection, MongoCollection, TinyCollection
 from .defines import ITERABLES, MOBILE_REGEX, MOBILE_PREFIX_REGEX, BODY_REGEX, TAG_REGEX,\
     EMAIL_REGEX, WINDOWS_LOCALE
@@ -85,7 +88,7 @@ from .exceptions import AppierException, OperationalError, SecurityError, Assert
 from .export import ExportManager, MongoEncoder
 from .geo import GeoResolver
 from .git import Git
-from .http import get, post, put, delete
+from .http import get_f, get, post, put, delete
 from .log import MemoryHandler, ThreadFormatter, rotating_handler, smtp_handler, in_signature
 from .meta import Ordered, Indexed
 from .model import Model, LocalModel, Field, link, operation, field
@@ -102,17 +105,17 @@ from .smtp import message, message_base, message_netius, smtp_engine, multipart,
     html, header
 from .structures import OrderedDict
 from .typesf import Type, File, Files, ImageFile, ImageFiles, image, images, Reference,\
-    reference, References, references
+    reference, References, references, Encrypted, encrypted, secure
 from .util import is_iterable, is_mobile, email_parts, email_mime, email_name, email_base,\
-    obfuscate, request_json, get_object, resolve_alias, page_types, find_types, norm_object,\
-    set_object, leafs, gen_token, html_to_text, camel_to_underscore, camel_to_readable, quote,\
-    unquote, base_name, base_name_m, parse_cookie, parse_multipart, decode_params, load_form,\
-    check_login, ensure_login, private, ensure, delayed, route, error_handler, exception_handler,\
-    before_request, after_request, is_detached, sanitize, verify, execute, ctx_locale, FileTuple,\
-    BaseThread
-from .validation import validate, validate_b, safe, eq, gt, gte, lt, lte, not_null, not_empty,\
-    not_false, is_in, is_simple, is_email, is_url, is_regex, field_eq, field_gt, field_gte,\
-    field_lt, field_lte, string_gt, string_lt, string_eq, equals, not_past, not_duplicate,\
+    date_to_timestamp, obfuscate, request_json, get_object, resolve_alias, page_types,\
+    find_types, norm_object, set_object, leafs, gen_token, html_to_text, camel_to_underscore,\
+    camel_to_readable, quote, unquote, base_name, base_name_m, parse_cookie, parse_multipart,\
+    decode_params, load_form, check_login, ensure_login, dict_merge, cached, private, ensure, delayed,\
+    route, error_handler, exception_handler, before_request, after_request, is_detached, sanitize,\
+    verify, execute, ctx_locale, FileTuple, BaseThread, JSONEncoder
+from .validation import validate, validate_b, validate_e, safe, eq, gt, gte, lt, lte, not_null,\
+    not_empty, not_false, is_in, is_simple, is_lower, is_email, is_url, is_regex, field_eq, field_gt,\
+    field_gte, field_lt, field_lte, string_gt, string_lt, string_eq, equals, not_past, not_duplicate,\
     all_different, no_self
 
 from .mongo import get_connection as get_mongo
