@@ -60,11 +60,21 @@ class Api(observer.Observable):
     with issues like: logging, building and destruction.
     """
 
+    SINGLETON = None
+
     def __init__(self, owner = None, *args, **kwargs):
         observer.Observable.__init__(self, *args, **kwargs)
         self.owner = owner or base.APP
         if not hasattr(self, "auth_callback"): self.auth_callback = None
 
+    @classmethod
+    def singleton(cls, *args, **kwargs):
+        callback = kwargs.pop("scallback", None)
+        if cls.SINGLETON: return cls.SINGLETON
+        cls.SINGLETON = cls(*args, **kwargs)
+        if callback: callback(cls.SINGLETON)
+        return cls.SINGLETON
+
     def get(
         self,
         url,
