@@ -46,6 +46,7 @@ import binascii
 import threading
 
 from . import mongo
+from . import common
 from . import config
 from . import legacy
 from . import exceptions
@@ -57,6 +58,10 @@ class DataAdapter(object):
         self._machine_bytes = self.__machine_bytes()
         self._inc_lock = threading.RLock()
 
+    @classmethod
+    def name_g(cls):
+        return cls.__name__[:-7].lower()
+
     def collection(self, name, *args, **kwargs):
         raise exceptions.NotImplementedError()
 
@@ -74,6 +79,11 @@ class DataAdapter(object):
             )
         return value
 
+    @property
+    def name(self):
+        cls = self.__class__
+        return cls.name_g()
+
     def _id(self):
         token = struct.pack(">i", int(time.time()))
         token += self._machine_bytes
@@ -100,7 +110,7 @@ class MongoAdapter(DataAdapter):
     def collection(self, name, *args, **kwargs):
         db = self.get_db()
         collection = db[name]
-        return MongoCollection(self, collection)
+        return MongoCollection(self, name, collection)
 
     def get_db(self):
         return mongo.get_db()
@@ -120,29 +130,51 @@ class TinyAdapter(DataAdapter):
     def __init__(self, *args, **kwargs):
         DataAdapter.__init__(self, *args, **kwargs)
         self.file_path = config.conf("TINY_PATH", "db.json")
+        self.storage = config.conf("TINY_STORAGE", "json")
         self.file_path = kwargs.get("file_path", self.file_path)
         self._db = None
 
     def collection(self, name, *args, **kwargs):
         db = self.get_db()
         table = db.table(name)
-        return TinyCollection(self, table)
+        return TinyCollection(self, name, table)
 
     def get_db(self):
-        import tinydb
-        if self._db: return self._db
-        self._db = tinydb.TinyDB(self.file_path)
+        if not self._db == None: return self._db
+        method = getattr(self, "_get_db_%s" % self.storage)
+        self._db = method()
         return self._db
 
     def drop_db(self, *args, **kwargs):
+        if self._db == None: return
         db = self.get_db()
         db.purge_tables()
-        os.remove
+        db.close()
+        self._db = None
+        method = getattr(self, "_drop_db_%s" % self.storage)
+        method()
+
+    def _get_db_json(self):
+        import tinydb
+        return tinydb.TinyDB(self.file_path)
+
+    def _get_db_memory(self):
+        import tinydb
+        return tinydb.TinyDB(
+            storage = tinydb.storages.MemoryStorage
+        )
+
+    def _drop_db_json(self):
+        os.remove(self.file_path)
+
+    def _drop_db_memory(self):
+        pass
 
 class Collection(object):
 
-    def __init__(self, owner):
+    def __init__(self, owner, name):
         self.owner = owner
+        self.name = name
 
     def find(self, *args, **kwargs):
         raise exceptions.NotImplementedError()
@@ -171,56 +203,80 @@ class Collection(object):
     def object_id(self, *args, **kwargs):
         return self.owner.object_id(*args, **kwargs)
 
+    def log(self, operation, *args, **kwargs):
+        show_queries = config.conf("SHOW_QUERIES", False, cast = bool)
+        if not show_queries: return
+        logger = common.base().get_logger()
+        extra = kwargs or args
+        logger.debug(
+            "[%s] %10s -> %12s <-> %s" %\
+            (self.owner.name, operation, self.name, str(extra)[:2046])
+        )
+
     def _id(self, *args, **kwargs):
         return self.owner._id(*args, **kwargs)
 
 class MongoCollection(Collection):
 
-    def __init__(self, owner, base):
-        Collection.__init__(self, owner)
+    def __init__(self, owner, name, base):
+        Collection.__init__(self, owner, name)
         self._base = base
 
     def find(self, *args, **kwargs):
+        self.log("find", *args, **kwargs)
         return self._base.find(*args, **kwargs)
 
     def find_one(self, *args, **kwargs):
+        self.log("find_one", *args, **kwargs)
         return self._base.find_one(*args, **kwargs)
 
     def find_and_modify(self, *args, **kwargs):
+        self.log("find_and_modify", *args, **kwargs)
         return mongo._store_find_and_modify(self._base, *args, **kwargs)
 
     def insert(self, *args, **kwargs):
+        self.log("insert", *args, **kwargs)
         return mongo._store_insert(self._base, *args, **kwargs)
 
     def update(self, *args, **kwargs):
+        self.log("update", *args, **kwargs)
         return mongo._store_update(self._base, *args, **kwargs)
 
     def remove(self, *args, **kwargs):
+        self.log("remove", *args, **kwargs)
         return mongo._store_remove(self._base, *args, **kwargs)
 
     def count(self, *args, **kwargs):
+        self.log("count", *args, **kwargs)
         return self._base.count(*args, **kwargs)
 
     def ensure_index(self, *args, **kwargs):
+        self.log("ensure_index", *args, **kwargs)
         return mongo._store_ensure_index(self._base, *args, **kwargs)
 
 class TinyCollection(Collection):
 
-    def __init__(self, owner, base):
-        Collection.__init__(self, owner)
+    def __init__(self, owner, name, base):
+        Collection.__init__(self, owner, name)
         self._base = base
 
     def find(self, *args, **kwargs):
+        self.log("find", *args, **kwargs)
         filter = args[0] if len(args) > 0 else dict()
         condition = self._to_condition(filter)
-        return self._base.search(condition)
+        results = self._base.search(condition)
+        return self._to_results(results, kwargs)
 
     def find_one(self, *args, **kwargs):
+        self.log("find_one", *args, **kwargs)
         filter = args[0] if len(args) > 0 else dict()
         condition = self._to_condition(filter)
-        return self._base.get(condition)
+        results = self._base.search(condition)
+        results = self._to_results(results, kwargs)
+        return results[0] if results else None
 
     def find_and_modify(self, *args, **kwargs):
+        self.log("find_and_modify", *args, **kwargs)
         filter = args[0] if len(args) > 0 else dict()
         modification = args[1] if len(args) > 1 else dict()
         create = kwargs.get("new", False)
@@ -233,11 +289,12 @@ class TinyCollection(Collection):
             )
         if not found: object = dict(filter)
         object = self._to_update(modification, object = object)
-        if found: self.update(object)
+        if found: self.update(filter, {"$set" : object})
         else: self.insert(object)
-        return object
+        return dict(object)
 
     def insert(self, *args, **kwargs):
+        self.log("insert", *args, **kwargs)
         object = args[0] if len(args) > 0 else dict()
         has_id = "_id" in object
         if not has_id: object["_id"] = self._id()
@@ -245,6 +302,7 @@ class TinyCollection(Collection):
         return object
 
     def update(self, *args, **kwargs):
+        self.log("update", *args, **kwargs)
         filter = args[0] if len(args) > 0 else dict()
         updater = args[1] if len(args) > 1 else dict()
         condition = self._to_condition(filter)
@@ -252,17 +310,19 @@ class TinyCollection(Collection):
         return self._base.update(object, condition)
 
     def remove(self, *args, **kwargs):
+        self.log("remove", *args, **kwargs)
         filter = args[0] if len(args) > 0 else dict()
         condition = self._to_condition(filter)
         return self._base.remove(condition)
 
     def count(self, *args, **kwargs):
+        self.log("count", *args, **kwargs)
         filter = args[0] if len(args) > 0 else dict()
         condition = self._to_condition(filter)
         return self._base.count(condition)
 
     def ensure_index(self, *args, **kwargs):
-        pass
+        self.log("ensure_index", *args, **kwargs)
 
     def _to_condition(self, filter):
         import tinydb
@@ -275,6 +335,24 @@ class TinyCollection(Collection):
             condition &= _condition
         return condition
 
+    def _to_results(self, results, kwargs, build = True):
+        sort = kwargs.get("sort", [])
+        skip = kwargs.get("skip", 0)
+        limit = kwargs.get("limit", None)
+
+        limit = None if limit == 0 else limit
+        reverse = sort[0][1] == -1 if sort else False
+
+        def sorter(value):
+            result = []
+            for item in sort: result.append(value[item[0]])
+            return tuple(result)
+
+        if sort: results.sort(key = sorter, reverse = reverse)
+        if skip or limit: results = results[slice(skip, skip + limit, 1)]
+        if build: results = [dict(result) for result in results]
+        return results
+
     def _to_update(self, modification, object = None):
         object = object or dict()
         increments = modification.get("$inc", {})
