@@ -317,6 +317,7 @@ class ExportManager(object):
 class MongoEncoder(json.JSONEncoder):
 
     def default(self, obj, **kwargs):
-        if not bson: return json.JSONEncoder.default(obj, **kwargs)
+        if not bson: return json.JSONEncoder.default(self, obj, **kwargs)
         if isinstance(obj, bson.objectid.ObjectId): return str(obj)
-        else: return json.JSONEncoder.default(obj, **kwargs)
+        if isinstance(obj, legacy.BYTES): return legacy.str(obj, encoding = "utf-8")
+        else: return json.JSONEncoder.default(self, obj, **kwargs)
