@@ -42,8 +42,12 @@ import base64
 import string
 import random
 import logging
+import threading
 
+from . import util
+from . import common
 from . import legacy
+from . import typesf
 from . import config
 from . import exceptions
 
@@ -70,13 +74,31 @@ def try_auth(auth_callback, params, headers = None):
     if headers == None: headers = dict()
     auth_callback(params, headers)
 
-def get(url, params = None, headers = None, handle = None, auth_callback = None):
+def get_f(*args, **kwargs):
+    name = kwargs.pop("name", "default")
+    kwargs["handle"] = kwargs.get("handle", True)
+    kwargs["redirect"] = kwargs.get("redirect", True)
+    data, response = get(*args, **kwargs)
+    info = response.info()
+    mime = info.get("Content-Type", None)
+    file_tuple = util.FileTuple((name, mime, data))
+    return typesf.File(file_tuple)
+
+def get(
+    url,
+    params = None,
+    headers = None,
+    handle = None,
+    redirect = None,
+    auth_callback = None
+):
     return _method(
         _get,
         url,
         params = params,
         headers = headers,
         handle = handle,
+        redirect = redirect,
         auth_callback = auth_callback
     )
 
@@ -89,6 +111,7 @@ def post(
     headers = None,
     mime = None,
     handle = None,
+    redirect = None,
     auth_callback = None
 ):
     return _method(
@@ -101,6 +124,7 @@ def post(
         headers = headers,
         mime = mime,
         handle = handle,
+        redirect = redirect,
         auth_callback = auth_callback
     )
 
@@ -113,6 +137,7 @@ def put(
     headers = None,
     mime = None,
     handle = None,
+    redirect = None,
     auth_callback = None
 ):
     return _method(
@@ -125,16 +150,25 @@ def put(
         headers = headers,
         mime = mime,
         handle = handle,
+        redirect = redirect,
         auth_callback = auth_callback
     )
 
-def delete(url, params = None, headers = None, handle = None, auth_callback = None):
+def delete(
+    url,
+    params = None,
+    headers = None,
+    handle = None,
+    redirect = None,
+    auth_callback = None
+):
     return _method(
         _delete,
         url,
         params = params,
         headers = headers,
         handle = handle,
+        redirect = redirect,
         auth_callback = auth_callback
     )
 
@@ -147,6 +181,7 @@ def patch(
     headers = None,
     mime = None,
     handle = None,
+    redirect = None,
     auth_callback = None
 ):
     return _method(
@@ -159,6 +194,7 @@ def patch(
         headers = headers,
         mime = mime,
         handle = handle,
+        redirect = redirect,
         auth_callback = auth_callback
     )
 
@@ -179,13 +215,20 @@ def _method(method, *args, **kwargs):
 
     return result
 
-def _get(url, params = None, headers = None, handle = None):
+def _get(
+    url,
+    params = None,
+    headers = None,
+    handle = None,
+    redirect = None
+):
     return _method_empty(
         "GET",
         url,
         params = params,
         headers = headers,
-        handle = handle
+        handle = handle,
+        redirect = redirect
     )
 
 def _post(
@@ -196,7 +239,8 @@ def _post(
     data_m = None,
     headers = None,
     mime = None,
-    handle = None
+    handle = None,
+    redirect = None
 ):
     return _method_payload(
         "POST",
@@ -207,7 +251,8 @@ def _post(
         data_m = data_m,
         headers = headers,
         mime = mime,
-        handle = handle
+        handle = handle,
+        redirect = redirect
     )
 
 def _put(
@@ -218,7 +263,8 @@ def _put(
     data_m = None,
     headers = None,
     mime = None,
-    handle = None
+    handle = None,
+    redirect = None
 ):
     return _method_payload(
         "PUT",
@@ -229,16 +275,24 @@ def _put(
         data_m = data_m,
         headers = headers,
         mime = mime,
-        handle = handle
+        handle = handle,
+        redirect = redirect
     )
 
-def _delete(url, params = None, headers = None, handle = None):
+def _delete(
+    url,
+    params = None,
+    headers = None,
+    handle = None,
+    redirect = None
+):
     return _method_empty(
         "DELETE",
         url,
         params = params,
         headers = headers,
-        handle = handle
+        handle = handle,
+        redirect = redirect
     )
 
 def _patch(
@@ -249,7 +303,8 @@ def _patch(
     data_m = None,
     headers = None,
     mime = None,
-    handle = None
+    handle = None,
+    redirect = None
 ):
     return _method_payload(
         "PATCH",
@@ -260,7 +315,8 @@ def _patch(
         data_m = data_m,
         headers = headers,
         mime = mime,
-        handle = handle
+        handle = handle,
+        redirect = redirect
     )
 
 def _method_empty(
@@ -269,15 +325,19 @@ def _method_empty(
     params = None,
     headers = None,
     handle = None,
+    redirect = None,
     timeout = TIMEOUT
 ):
     if handle == None: handle = False
+    if redirect == None: redirect = False
     values = params or dict()
 
     logging.info("%s %s with '%s'" % (name, url, str(values)))
 
+    url, scheme, host, authorization, extra = _parse_url(url)
+    if extra: values.update(extra)
     data = _urlencode(values)
-    url, host, authorization = _parse_url(url)
+
     headers = headers or dict()
     if host: headers["Host"] = host
     if authorization: headers["Authorization"] = "Basic %s" % authorization
@@ -291,6 +351,9 @@ def _method_empty(
     code = file.getcode()
     info = file.info()
 
+    location = info.get("Location", None) if redirect else None
+    if location: return _redirect(location, scheme, host, handle, redirect)
+
     logging.info("%s %s returned '%d'" % (name, url, code))
 
     result = _result(result, info)
@@ -306,14 +369,17 @@ def _method_payload(
     headers = None,
     mime = None,
     handle = None,
+    redirect = None,
     timeout = TIMEOUT
 ):
     if handle == None: handle = False
+    if redirect == None: redirect = False
     values = params or dict()
 
     logging.info("%s %s with '%s'" % (name, url, str(params)))
 
-    url, host, authorization = _parse_url(url)
+    url, scheme, host, authorization, extra = _parse_url(url)
+    if extra: values.update(extra)
     data_e = _urlencode(values)
 
     if not data == None:
@@ -349,11 +415,23 @@ def _method_payload(
     code = file.getcode()
     info = file.info()
 
+    location = info.get("Location", None) if redirect else None
+    if location: return _redirect(location, scheme, host, handle, redirect)
+
     logging.info("%s %s returned '%d'" % (name, url, code))
 
     result = _result(result, info)
     return (result, file) if handle else result
 
+def _redirect(location, scheme, host, handle, redirect):
+    is_relative = location.startswith("/")
+    if is_relative: location = scheme + "://" + host + location
+    return get(
+        location,
+        handle = handle,
+        redirect = redirect
+    )
+
 def _resolve(*args, **kwargs):
     _global = globals()
     client = config.conf("HTTP_CLIENT", "netius")
@@ -363,33 +441,93 @@ def _resolve(*args, **kwargs):
     except ImportError: result = _resolve_legacy(*args, **kwargs)
     return result
 
-def _resolve_legacy(url, method, headers, data, timeout):
+def _resolve_legacy(url, method, headers, data, timeout, **kwargs):
     opener = legacy.build_opener(legacy.HTTPHandler)
     request = legacy.Request(url, data = data, headers = headers)
     request.get_method = lambda: method
     return opener.open(request, timeout = timeout)
 
-def _resolve_netius(url, method, headers, data, timeout):
+def _resolve_netius(url, method, headers, data, timeout, **kwargs):
     import netius.clients
+
+    # converts the provided dictionary of headers into a new map to
+    # allow any re-writing of values, valuable for a re-connect
     headers = dict(headers)
+
+    # retrieves the various dynamic parameters for the http client
+    # usage under the netius infra-structure
+    retry = kwargs.get("retry", 1)
+    reuse = kwargs.get("reuse", True)
+
+    # verifies if client re-usage must be enforced and if that's the
+    # case the global client object is requested (singleton) otherwise
+    # the client should be created inside the http client static method
+    http_client = _client_netius() if reuse else None
     result = netius.clients.HTTPClient.method_s(
         method,
         url,
         headers = headers,
         data = data,
-        async = False
+        async = False,
+        http_client = http_client
     )
+
+    # tries to retrieve any possible error coming from the result object
+    # if this happens it means an exception has been raised internally and
+    # the error should be handled in a proper manner, if the error is related
+    # to a closed connection a retry may be performed to try to re-establish
+    # the connection (allows for reconnection in connection pool)
+    error = result.get("error", None)
+    if error == "closed":
+        if retry < 1: raise exceptions.OperationalError(message = "Connection closed")
+        kwargs["retry"] = retry - 1
+        return _resolve_netius(
+            url, method, headers, data, timeout, **kwargs
+        )
+
+    # converts the netius specific result map into a response compatible
+    # object (equivalent to the urllib one) to be used by the upper layers
+    # under an equivalent and compatible approach
     response = netius.clients.HTTPClient.to_response(result)
+
+    # retrieves the response code of the created response and verifies if
+    # it represent an error, if that's the case raised an error exception
+    # to the upper layers to break the current execution logic properly
     code = response.getcode()
     is_error = code // 100 in (4, 5) if code else True
     if is_error: raise legacy.HTTPError(
         url, code, "HTTP retrieval problem", None, response
     )
+
+    # returns the final response object to the upper layers, this object
+    # may be used freely under the compatibility interface it provides
     return response
 
+def _client_netius():
+    import netius.clients
+    global _netius_clients
+    registered = "_netius_clients" in globals()
+    _netius_clients = _netius_clients if registered else dict()
+    tid = threading.current_thread().ident
+    netius_client = _netius_clients.get(tid, None)
+    if netius_client: return netius_client
+    netius_client = netius.clients.HTTPClient(
+        thread = False,
+        auto_pause = True
+    )
+    _netius_clients[tid] = netius_client
+    if not registered: common.base().on_exit(_cleanup_netius)
+    return netius_client
+
+def _cleanup_netius():
+    global _netius_clients
+    for netius_client in _netius_clients.values(): netius_client.cleanup()
+    del _netius_clients
+
 def _parse_url(url):
     parse = legacy.urlparse(url)
-    secure = parse.scheme == "https"
+    scheme = parse.scheme
+    secure = scheme == "https"
     default = 443 if secure else 80
     port = parse.port or default
     url = parse.scheme + "://" + parse.hostname + ":" + str(port) + parse.path
@@ -397,13 +535,9 @@ def _parse_url(url):
     else: host = parse.hostname + ":" + str(port)
     username = parse.username
     password = parse.password
-    if username and password:
-        payload = "%s:%s" % (username, password)
-        payload = legacy.bytes(payload)
-        authorization = base64.b64encode(payload)
-        authorization = legacy.str(authorization)
-    else: authorization = None
-    return (url, host, authorization)
+    authorization = _authorization(username, password)
+    params = _params(parse.query)
+    return (url, scheme, host, authorization, params)
 
 def _result(data, info = {}, force = False, strict = False):
     # tries to retrieve the content type value from the headers
@@ -427,6 +561,35 @@ def _result(data, info = {}, force = False, strict = False):
         if strict: raise
     return data
 
+def _params(query):
+    # creates the dictionary that is going to be used to store the
+    # complete information regarding the parameters in query
+    params = dict()
+
+    # validates that the provided query value is valid and if
+    # that's not the case returns the created parameters immediately
+    # (empty parameters are returned)
+    if not query: return params
+
+    # splits the query value around the initial parameter separator
+    # symbol and iterates over each of them to parse them and create
+    # the proper parameters dictionary (of lists)
+    query_s = query.split("&")
+    for part in query_s:
+        parts = part.split("=", 1)
+        if len(parts) == 1: value = ""
+        else: value = parts[1]
+        key = parts[0]
+        key = legacy.unquote_plus(key)
+        value = legacy.unquote_plus(value)
+        param = params.get(key, [])
+        param.append(value)
+        params[key] = param
+
+    # returns the final parameters dictionary to the caller method
+    # so that it may be used as a proper structure representation
+    return params
+
 def _urlencode(values, as_string = True):
     # creates the dictionary that will hold the final
     # dictionary of values (without the unset and
@@ -492,6 +655,15 @@ def _quote(values, plus = False, safe = "/"):
 
     return final
 
+def _authorization(username, password):
+    if not username: return None
+    if not password: return None
+    payload = "%s:%s" % (username, password)
+    payload = legacy.bytes(payload)
+    authorization = base64.b64encode(payload)
+    authorization = legacy.str(authorization)
+    return authorization
+
 def _encode_multipart(fields, mime = None, doseq = False):
     mime = mime or "multipart/form-data"
     boundary = _create_boundary(fields, doseq = doseq)
@@ -504,9 +676,8 @@ def _encode_multipart(fields, mime = None, doseq = False):
         values = values if is_list else [values]
 
         for value in values:
-            value_t = type(value)
 
-            if value_t == dict:
+            if isinstance(value, dict):
                 header_l = []
                 data = None
                 for key, item in value.items():
@@ -514,10 +685,14 @@ def _encode_multipart(fields, mime = None, doseq = False):
                     else: header_l.append("%s: %s" % (key, item))
                 value = data
                 header = "\r\n".join(header_l)
-            elif value_t == tuple:
+            elif isinstance(value, tuple):
+                content_type = None
+                if len(value) == 2: name, contents = value
+                else: name, content_type, contents = value
                 header = "Content-Disposition: form-data; name=\"%s\"; filename=\"%s\"" %\
-                    (key, value[0])
-                value = value[1]
+                    (key, name)
+                if content_type: header += "\r\nContent-Type: %s" % content_type
+                value = contents
             else:
                 header = "Content-Disposition: form-data; name=\"%s\"" % key
                 value = _encode(value)
