@@ -43,6 +43,7 @@ import sys
 import inspect
 import functools
 import itertools
+import contextlib
 import collections
 
 import urllib #@UnusedImport
@@ -52,35 +53,35 @@ ArgSpec = collections.namedtuple(
     ["args", "varargs", "keywords", "defaults"]
 )
 
-root = sys.path.pop(0)
-try: import urllib2
-except ImportError: urllib2 = None
-finally: sys.path.insert(0, root)
+@contextlib.contextmanager
+def ctx_absolute():
+    root = sys.path.pop(0)
+    try: yield
+    finally: sys.path.insert(0, root)
 
-root = sys.path.pop(0)
-try: import httplib
-except ImportError: httplib = None
-finally: sys.path.insert(0, root)
+with ctx_absolute():
+    try: import urllib2
+    except ImportError: urllib2 = None
 
-root = sys.path.pop(0)
-try: import http
-except ImportError: http = None
-finally: sys.path.insert(0, root)
+with ctx_absolute():
+    try: import httplib
+    except ImportError: httplib = None
 
-root = sys.path.pop(0)
-try: import urllib.error
-except ImportError: pass
-finally: sys.path.insert(0, root)
+with ctx_absolute():
+    try: import http
+    except ImportError: http = None
 
-root = sys.path.pop(0)
-try: import urllib.request
-except ImportError: pass
-finally: sys.path.insert(0, root)
+with ctx_absolute():
+    try: import urllib.error
+    except ImportError: pass
 
-root = sys.path.pop(0)
-try: import http.client
-except ImportError: pass
-finally: sys.path.insert(0, root)
+with ctx_absolute():
+    try: import urllib.request
+    except ImportError: pass
+
+with ctx_absolute():
+    try: import http.client
+    except ImportError: pass
 
 try: import HTMLParser
 except ImportError: import html.parser; HTMLParser = html.parser
@@ -102,6 +103,10 @@ PYTHON_3 = sys.version_info[0] >= 3
 interpreter is at least python 3 compliant, this is used
 to take some of the conversion decision for runtime """
 
+PYTHON_V = int("".join([str(v) for v in sys.version_info[:3]]))
+""" The python version integer describing the version of
+a the interpreter as a set of three integer digits """
+
 if PYTHON_3: LONG = int
 else: LONG = long #@UndefinedVariable
 
@@ -117,6 +122,9 @@ else: OLD_UNICODE = unicode #@UndefinedVariable
 if PYTHON_3: STRINGS = (str,)
 else: STRINGS = (str, unicode) #@UndefinedVariable
 
+if PYTHON_3: ALL_STRINGS = (bytes, str)
+else: ALL_STRINGS = (bytes, str, unicode) #@UndefinedVariable
+
 if PYTHON_3: INTEGERS = (int,)
 else: INTEGERS = (int, long) #@UndefinedVariable
 
@@ -140,10 +148,10 @@ else: HTTPHandler = urllib2.HTTPHandler
 if PYTHON_3: HTTPError = urllib.error.HTTPError
 else: HTTPError = urllib2.HTTPError
 
-if PYTHON_3: HTTPConnection = http.client.HTTPConnection
+if PYTHON_3: HTTPConnection = http.client.HTTPConnection #@UndefinedVariable
 else: HTTPConnection = httplib.HTTPConnection
 
-if PYTHON_3: HTTPSConnection = http.client.HTTPSConnection
+if PYTHON_3: HTTPSConnection = http.client.HTTPSConnection #@UndefinedVariable
 else: HTTPSConnection = httplib.HTTPSConnection
 
 try: _execfile = execfile #@UndefinedVariable
@@ -211,17 +219,17 @@ def chri(value):
     if type(value) in INTEGERS: return _chr(value)
     return value
 
-def bytes(value):
+def bytes(value, encoding = "latin-1"):
     if not PYTHON_3: return value
     if value == None: return value
     if type(value) == _bytes: return value
-    return value.encode("latin-1")
+    return value.encode(encoding)
 
-def str(value):
+def str(value, encoding = "latin-1"):
     if not PYTHON_3: return value
     if value == None: return value
     if type(value) == _str: return value
-    return value.decode("latin-1")
+    return value.decode(encoding)
 
 def orderable(value):
     if not PYTHON_3: return value
@@ -242,6 +250,10 @@ def is_bytes(value):
     if PYTHON_3: return type(value) == _bytes
     else: return type(value) == _str #@UndefinedVariable
 
+def is_string(value, all = False):
+    target = ALL_STRINGS if all else STRINGS
+    return type(value) in target
+
 def is_generator(value):
     if inspect.isgenerator(value): return True
     if type(value) in (itertools.chain,): return True
