@@ -133,19 +133,21 @@ class Indexed(type):
                 )
 
             for error in errors:
-                code, json = error
+                code, scope, json = error
                 app.add_error(
                     code,
                     function,
+                    scope = scope,
                     json = json,
                     context = new_name
                 )
 
             for exception in exceptions:
-                exception, json = exception
+                exception, scope, json = exception
                 app.add_exception(
                     exception,
                     function,
+                    scope = scope,
                     json = json,
                     context = new_name
                 )
