@@ -42,12 +42,12 @@ import copy
 import math
 import json
 import types
+import inspect
 import logging
 import datetime
 
 from . import meta
 from . import util
-from . import mongo
 from . import legacy
 from . import common
 from . import typesf
@@ -90,8 +90,8 @@ an extension to the base builder map """
 METAS = dict(
     text = lambda v, d: v,
     enum = lambda v, d: d["enum"].get(v, None),
-    list = lambda v, d: mongo.dumps(v),
-    map = lambda v, d: mongo.dumps(v),
+    list = lambda v, d: json.dumps(v),
+    map = lambda v, d: json.dumps(v),
     date = lambda v, d: datetime.datetime.utcfromtimestamp(float(v)).strftime("%d %b %Y"),
     datetime = lambda v, d: datetime.datetime.utcfromtimestamp(float(v)).strftime("%d %b %Y %H:%M:%S")
 )
@@ -238,7 +238,10 @@ class Model(legacy.with_meta(meta.Ordered, observer.Observable)):
         return instance
 
     def __init__(self, model = None, **kwargs):
-        self.__dict__["model"] = model or {}
+        fill = kwargs.pop("fill", True)
+        model = model or {}
+        if fill: model = self.__class__.fill(model)
+        self.__dict__["model"] = model
         self.__dict__["owner"] = common.base().APP or None
         self.__dict__["ref"] = kwargs.pop("ref", None)
         for name, value in kwargs.items(): setattr(self, name, value)
@@ -366,8 +369,8 @@ class Model(legacy.with_meta(meta.Ordered, observer.Observable)):
         """
 
         if model == None: model = util.get_object() if form else dict(kwargs)
-        if fill: model = cls.fill(model)
-        instance = cls()
+        if fill: model = cls.fill(model, safe = not new)
+        instance = cls(fill = False)
         instance.apply(model, form = form, safe_a = safe)
         build and cls.build(instance.model, map = False)
         new and instance.assert_is_new()
@@ -420,12 +423,13 @@ class Model(legacy.with_meta(meta.Ordered, observer.Observable)):
         if not is_sequence: models = [models]
         wrapping = []
         for model in models:
+            if not isinstance(model, dict): continue
             _model = cls(model = model, **kwargs)
             handler and handler(_model.model)
             build and cls.build(_model.model, map = False)
             wrapping.append(_model)
         if is_sequence: return wrapping
-        else: return wrapping[0]
+        else: return wrapping[0] if wrapping else None
 
     @classmethod
     def singleton(
@@ -451,9 +455,10 @@ class Model(legacy.with_meta(meta.Ordered, observer.Observable)):
 
     @classmethod
     def get(cls, *args, **kwargs):
-        fields, eager, map, rules, meta, build, fill, skip, limit, sort, raise_e = cls._get_attrs(kwargs, (
+        fields, eager, eager_l, map, rules, meta, build, fill, skip, limit, sort, raise_e = cls._get_attrs(kwargs, (
             ("fields", None),
             ("eager", None),
+            ("eager_l", None),
             ("map", False),
             ("rules", True),
             ("meta", False),
@@ -465,6 +470,8 @@ class Model(legacy.with_meta(meta.Ordered, observer.Observable)):
             ("raise_e", True)
         ))
 
+        if eager_l == None: eager_l = map
+        if eager_l: eager = cls._eager_b(eager)
         fields = cls._sniff(fields, rules = rules)
         collection = cls._collection()
         model = collection.find_one(
@@ -484,16 +491,18 @@ class Model(legacy.with_meta(meta.Ordered, observer.Observable)):
             )
         if not model and not raise_e: return model
         cls.types(model)
-        if fill: cls.fill(model)
+        if fill: cls.fill(model, safe = True)
         if build: cls.build(model, map = map, rules = rules, meta = meta)
         if eager: model = cls._eager(model, eager)
+        if map: model = cls._resolve_all(model, resolve = False)
         return model if map else cls.old(model = model, safe = False)
 
     @classmethod
     def find(cls, *args, **kwargs):
-        fields, eager, map, rules, meta, build, fill, skip, limit, sort = cls._get_attrs(kwargs, (
+        fields, eager, eager_l, map, rules, meta, build, fill, skip, limit, sort = cls._get_attrs(kwargs, (
             ("fields", None),
             ("eager", None),
+            ("eager_l", False),
             ("map", False),
             ("rules", True),
             ("meta", False),
@@ -504,6 +513,7 @@ class Model(legacy.with_meta(meta.Ordered, observer.Observable)):
             ("sort", None)
         ))
 
+        if eager_l: eager = cls._eager_b(eager)
         cls._find_s(kwargs)
         cls._find_d(kwargs)
 
@@ -517,9 +527,10 @@ class Model(legacy.with_meta(meta.Ordered, observer.Observable)):
             sort = sort
         )
         models = [cls.types(model) for model in models]
-        if fill: models = [cls.fill(model) for model in models]
+        if fill: models = [cls.fill(model, safe = True) for model in models]
         if build: [cls.build(model, map = map, rules = rules, meta = meta) for model in models]
         if eager: models = cls._eager(models, eager)
+        if map: models = [cls._resolve_all(model, resolve = False) for model in models]
         models = models if map else [cls.old(model = model, safe = False) for model in models]
         return models
 
@@ -870,6 +881,10 @@ class Model(legacy.with_meta(meta.Ordered, observer.Observable)):
         collection = cls._collection()
         for index in indexes: collection.ensure_index(index)
 
+    @classmethod
+    def teardown(cls):
+        pass
+
     @classmethod
     def validate(cls):
         return []
@@ -916,6 +931,10 @@ class Model(legacy.with_meta(meta.Ordered, observer.Observable)):
     def extra_names(cls):
         return []
 
+    @classmethod
+    def unique_names(cls):
+        return []
+
     @classmethod
     def order_name(self):
         return None
@@ -947,9 +966,9 @@ class Model(legacy.with_meta(meta.Ordered, observer.Observable)):
         return model
 
     @classmethod
-    def fill(cls, model = None):
+    def fill(cls, model = None, safe = False):
         """
-        Fills the current models with the proper values so that
+        Fills the current model with the proper values so that
         no values are unset as this would violate the model definition
         integrity. This is required when retrieving an object(s) from
         the data source (as some of them may be incomplete).
@@ -958,6 +977,10 @@ class Model(legacy.with_meta(meta.Ordered, observer.Observable)):
         :param model: The model that is going to have its unset
         attributes filled with "default" data, in case none is provided
         all of the attributes will be filled with "default" data.
+        :type safe: bool
+        :param safe: If the safe mode should be used for the fill
+        operation meaning that under some conditions no unit fill
+        operation is going to be applied (eg: retrieval operations).
         """
 
         model = model or dict()
@@ -965,7 +988,9 @@ class Model(legacy.with_meta(meta.Ordered, observer.Observable)):
         for name, _definition in definition.items():
             if name in model: continue
             if name in ("_id",): continue
+            private = _definition.get("private", False)
             increment = _definition.get("increment", False)
+            if private and safe: continue
             if increment: continue
             if "initial" in _definition:
                 initial = _definition.get("initial")
@@ -1154,6 +1179,33 @@ class Model(legacy.with_meta(meta.Ordered, observer.Observable)):
         cls._immutables = immutables
         return immutables
 
+    @classmethod
+    def eagers(cls):
+        # in case the eagers are already "cached" in the current
+        # class (fast retrieval) returns immediately
+        if "_eagers" in cls.__dict__: return cls._eagers
+
+        # creates the list that will hold the various names that are
+        # meant to be eager values in the data source
+        eagers = []
+
+        # retrieves the map containing the definition of the class with
+        # the name of the fields associated with their definition
+        definition = cls.definition()
+
+        # iterate over all the names in the definition to retrieve their
+        # definition and check if their are of type eager
+        for name in definition:
+            _definition = cls.definition_n(name)
+            is_eager = _definition.get("eager", False)
+            if not is_eager: continue
+            eagers.append(name)
+
+        # saves the eagers list under the class and then
+        # returns the sequence to the caller method
+        cls._eagers = eagers
+        return eagers
+
     @classmethod
     def default(cls):
         # in case the default are already "cached" in the current
@@ -1293,10 +1345,11 @@ class Model(legacy.with_meta(meta.Ordered, observer.Observable)):
 
     @classmethod
     def _res(cls, model, part):
+        if not model: return model
         value = model[part]
         is_reference = isinstance(value, TYPE_REFERENCES)
         if not value and not is_reference: return value
-        if is_reference: model[part] = value.resolve()
+        if is_reference: model[part] = value.resolve(eager_l = True)
         model = model[part]
         return model
 
@@ -1323,6 +1376,10 @@ class Model(legacy.with_meta(meta.Ordered, observer.Observable)):
 
     @classmethod
     def _find_s(cls, kwargs):
+        # tries to retrieve the find name value from the provided
+        # named arguments defaulting to an unset value otherwise
+        find_n = kwargs.pop("find_n", None)
+
         # retrieves the kind of insensitive strategy that is going
         # to be used for the resolution of regular expressions,
         # this should affect all the filters and so it should be
@@ -1359,7 +1416,7 @@ class Model(legacy.with_meta(meta.Ordered, observer.Observable)):
         # to be the default (representation) for the model in case
         # there's none returns immediately, as it's not possible
         # to proceed with the filter creation
-        default = cls.default()
+        default = find_n or cls.default()
         if not default: return
 
         # constructs the proper right and left parts of the regex
@@ -1522,6 +1579,64 @@ class Model(legacy.with_meta(meta.Ordered, observer.Observable)):
         })
         return value["seq"]
 
+    @classmethod
+    def _eager_b(cls, eager):
+        """
+        Builds the provided list of eager values, preparing them
+        according to the current model rules.
+
+        The composition process includes the extension of the provided
+        sequence of eager values with the base ones defined in the
+        model, if not handled correctly this is an expensive operation.
+
+        :type eager: List/Tuple
+        :param eager: The base sequence containing the various fields
+        that should be eagerly loaded for the operation.
+        :rtype: Tuple
+        :return: The "final" resolved tuple that may be used for the eager
+        loaded operation performance.
+        """
+
+        eager = list(eager) if eager else []
+        eagers = cls.eagers()
+        eager.extend(eagers)
+        if not eager: return eager
+        eager = tuple(set(eager))
+        return eager
+
+    @classmethod
+    def _resolve_all(cls, model, *args, **kwargs):
+        definition = cls.definition()
+        for name, value in legacy.eager(model.items()):
+            if not name in definition: continue
+            model[name] = cls._resolve(name, value, *args, **kwargs)
+        return model
+
+    @classmethod
+    def _resolve(cls, name, value, *args, **kwargs):
+        # verifies if the current value is an iterable one in case
+        # it is runs the evaluate method for each of the values to
+        # try to resolve them into the proper representation
+        is_iterable = type(value) in (list, tuple)
+        if is_iterable: return [cls._resolve(name, value, *args, **kwargs) for value in value]
+
+        # verifies if the map value recursive approach should be used
+        # for the element and if that's the case calls the proper method
+        # otherwise uses the provided (raw value)
+        if not hasattr(value, "map_v"): return value
+        return value.map_v(*args, **kwargs)
+
+    @classmethod
+    def _to_meta(cls, base):
+        if type(base) == str: return base
+        is_class = inspect.isclass(type)
+        if not is_class: base = base.__class__
+        for cls in base.mro():
+            result = TYPE_META.get(cls, None)
+            if not result: continue
+            return result
+        return base
+
     @property
     def request(self):
         return common.base().get_request()
@@ -1538,6 +1653,13 @@ class Model(legacy.with_meta(meta.Ordered, observer.Observable)):
     def val(self, name, default = None):
         return self.model.get(name, default)
 
+    def json_v(self, *args, **kwargs):
+        return self.model
+
+    def map_v(self, *args, **kwargs):
+        resolve = kwargs.get("resolve", True)
+        return self._resolve_all(self.model, resolve = resolve)
+
     def build_m(self, model = None, rules = True):
         """
         Builds the currently defined model, this should run
@@ -1551,7 +1673,7 @@ class Model(legacy.with_meta(meta.Ordered, observer.Observable)):
         This method should me used carefully to avoid validation
         problems and other side effects.
 
-        :type model: Map
+        :type model: Dictionary
         :param model: The model map to be used for the build
         operation in case none is specified the currently set
         model is used instead.
@@ -1613,8 +1735,8 @@ class Model(legacy.with_meta(meta.Ordered, observer.Observable)):
         cls = self.__class__
         model = self.model
         if not reset: return cls(model = model)
-        indexes = cls.indexes()
-        indexes = indexes + ["_id"]
+        indexes = cls.increments()
+        indexes = indexes + cls.unique_names() + ["_id"]
         for index in indexes:
             if not index in model: continue
             del model[index]
@@ -1660,10 +1782,19 @@ class Model(legacy.with_meta(meta.Ordered, observer.Observable)):
             code = 412
         )
 
-    def save(self, validate = True):
+    def save(
+        self,
+        validate = True,
+        is_new = None,
+        increment_a = None,
+        immutables_a = None
+    ):
         # checks if the instance to be saved is a new instance
-        # or if this is an update operation
-        is_new = self.is_new()
+        # or if this is an update operation and then determines
+        # series of default values taking that into account
+        if is_new == None: is_new = self.is_new()
+        if increment_a == None: increment_a = is_new
+        if immutables_a == None: immutables_a = not is_new
 
         # runs the validation process in the current model, this
         # should ensure that the model is ready to be saved in the
@@ -1685,8 +1816,8 @@ class Model(legacy.with_meta(meta.Ordered, observer.Observable)):
         # any relation is loaded the reference value is returned instead
         # of the loaded relation values (required for persistence)
         model = self._filter(
-            increment_a = is_new,
-            immutables_a = not is_new,
+            increment_a = increment_a,
+            immutables_a = immutables_a,
             normalize = True
         )
 
@@ -1755,12 +1886,23 @@ class Model(legacy.with_meta(meta.Ordered, observer.Observable)):
         cls = self.__class__
         return cls.get(_id = self._id, *args, **kwargs)
 
-    def map(self, resolve = False, all = False):
-        model = self._filter(resolve = resolve, all = all)
+    def exists(self):
+        is_new = self.is_new()
+        if is_new: return False
+        entity = self.get(_id = self._id, raise_e = False)
+        return True if entity else False
+
+    def map(self, increment_a = False, resolve = False, all = False):
+        model = self._filter(
+            increment_a = increment_a,
+            resolve = resolve,
+            all = all,
+            evaluator = "map_v"
+        )
         return model
 
     def dumps(self):
-        return mongo.dumps(self.model)
+        return json.dumps(self.model)
 
     def unwrap(self, **kwargs):
         default = kwargs.get("default", False)
@@ -1875,7 +2017,8 @@ class Model(legacy.with_meta(meta.Ordered, observer.Observable)):
         immutables_a = False,
         normalize = False,
         resolve = False,
-        all = False
+        all = False,
+        evaluator = "json_v"
     ):
         # creates the model that will hold the "filtered" model
         # with all the items that conform with the class specification
@@ -1911,7 +2054,7 @@ class Model(legacy.with_meta(meta.Ordered, observer.Observable)):
         for name, value in legacy.eager(self.model.items()):
             if not name in definition: continue
             if immutables_a and name in immutables: continue
-            value = self._evaluate(name, value)
+            value = self._evaluate(name, value, evaluator = evaluator)
             model[name] = value
 
         # in case the normalize flag is set must iterate over all
@@ -1932,7 +2075,7 @@ class Model(legacy.with_meta(meta.Ordered, observer.Observable)):
         if resolve:
             for name, value in legacy.eager(self.model.items()):
                 if not name in definition: continue
-                model[name] = self._resolve(name, value)
+                model[name] = cls._resolve(name, value)
 
         # in case the all flag is set the extra fields (not present
         # in definition) must also be used to populate the resulting
@@ -1947,13 +2090,20 @@ class Model(legacy.with_meta(meta.Ordered, observer.Observable)):
         # from the validation of the items against the model class
         return model
 
-    def _evaluate(self, name, value):
+    def _evaluate(self, name, value, evaluator = "json_v"):
         # verifies if the current value is an iterable one in case
         # it is runs the evaluate method for each of the values to
-        # try to resolve them into the proper representation
+        # try to resolve them into the proper representation, note
+        # that both base iterable values (lists and dictionaries) and
+        # objects that implement the evaluator method are not considered
+        # to be iterables and normal operation applies
         is_iterable = hasattr(value, "__iter__")
-        is_iterable = is_iterable and not type(value) in ITERABLES
-        if is_iterable: return [self._evaluate(name, value) for value in value]
+        is_iterable = is_iterable and not type(value) in ITERABLES and\
+           not hasattr(value, evaluator)
+        if is_iterable: return [
+            self._evaluate(name, value, evaluator = evaluator) for\
+            value in value
+        ]
 
         # verifies the current value's class is sub class of the model
         # class and in case it's extracts the relation name from the
@@ -1965,26 +2115,13 @@ class Model(legacy.with_meta(meta.Ordered, observer.Observable)):
             _name = _type._name
             value = getattr(value, _name)
 
-        # iterates over all the values and retrieves the json value for
-        # each of them in case the value contains a json value retrieval
+        # iterates over all the values and retrieves the map value for
+        # each of them in case the value contains a map value retrieval
         # method otherwise uses the normal value returning it to the caller
-        value = value.json_v() if hasattr(value, "json_v") else value
+        method = getattr(value, evaluator) if hasattr(value, evaluator) else None
+        value = method(resolve = False) if method else value
         return value
 
-    def _resolve(self, name, value, all = False):
-        # verifies if the current value is an iterable one in case
-        # it is runs the evaluate method for each of the values to
-        # try to resolve them into the proper representation
-        is_iterable = hasattr(value, "__iter__")
-        is_iterable = is_iterable and not type(value) in ITERABLES
-        if is_iterable: return [self._resolve(name, value) for value in value]
-
-        # verifies if the map value recursive approach should be used
-        # for the element and if that's the case calls the proper method
-        # otherwise uses the provided (raw value)
-        if not hasattr(value, "map_v"): return value
-        return value.map_v(resolve = True, all = all)
-
 class LocalModel(Model):
     """
     Concrete model aimed at cases where data source based
@@ -2052,11 +2189,12 @@ class Action(dict):
         # various "passed" raw values
         parameters = self.get("parameters", [])
         for value, parameters in zip(values, parameters):
-            cast = parameters[2]
+            type = parameters[2]
+            cast = type
             is_default = value in (None, "")
             cast = BUILDERS.get(cast, cast)
             if cast and not is_default: value = cast(value)
-            if is_default: value = TYPE_DEFAULTS.get(cast, value)
+            if is_default: value = TYPE_DEFAULTS.get(type, value)
             casted.append(value)
 
         # returns the final list of casted values to the caller method
@@ -2111,7 +2249,13 @@ def link(name = None, parameters = ()):
 
     return decorator
 
-def operation(name = None, parameters = (), factory = False, level = 1):
+def operation(
+    name = None,
+    parameters = (),
+    factory = False,
+    level = 1,
+    devel = False
+):
     """
     Decorator function to be used to "annotate" the provided
     function as an operation that is able to change the current
@@ -2126,13 +2270,16 @@ def operation(name = None, parameters = (), factory = False, level = 1):
     :type parameters: Tuple
     :param parameters: The sequence containing tuples that describe
     the various parameters to be send to the operation.
+    :type factory: bool
+    :param factory: If the operation is considered to be a factory
+    meaning that a new entity is going to be created and/or updated.
     :type level: int
     :param level: The severity level of the operation, the higher
     values will be considered more severe than the lower ones,
     evaluation of the value will be context based.
-    :type factory: bool
-    :param factory: If the operation is considered to be a factory
-    meaning that a new entity is going to be created and/or updated.
+    :type devel: bool
+    :param devel: If the operation should only be used/available under
+    development like environments (eg: debugging purposes).
     :rtype: Function
     :return: The decorator function that is going to be used to
     generated the final function to be called.
@@ -2144,7 +2291,8 @@ def operation(name = None, parameters = (), factory = False, level = 1):
             name = name or function.__name__,
             parameters = parameters,
             factory = factory,
-            level = level
+            level = level,
+            devel = devel
         )
 
         return function
