@@ -72,14 +72,18 @@ class Session(object):
         name = "session",
         expire = EXPIRE_TIME,
         sid = None,
-        address = None
+        address = None,
+        growing = True
     ):
         object.__init__(self)
         self.sid = sid if sid else self._gen_sid()
         self.name = name
         self.address = address
         self.create = time.time()
-        self.expire = self.create + self._to_seconds(expire)
+        self.modify = self.create
+        self.duration = self._to_seconds(expire)
+        self.expire = self.create + self.duration
+        self.growing = growing
         self.dirty = True
         self.transient = dict()
 
@@ -90,10 +94,12 @@ class Session(object):
         return self.transient.__getitem__(key)
 
     def __setitem__(self, key, value):
-        self.mark(); self.transient.__setitem__(key, value)
+        self.mark(extend = self.growing)
+        self.transient.__setitem__(key, value)
 
     def __delitem__(self, key):
-        self.mark(); self.transient.__delitem__(key)
+        self.mark(extend = self.growing)
+        self.transient.__delitem__(key)
 
     def __iter__(self):
         return self.transient.__iter__()
@@ -113,7 +119,10 @@ class Session(object):
             name = self.name,
             address = self.address,
             create = self.create,
+            modify = self.modify,
+            duration = self.duration,
             expire = self.expire,
+            growing = self.growing,
             dirty = self.dirty
         )
 
@@ -123,10 +132,14 @@ class Session(object):
             "name",
             "address",
             "create",
+            "modify",
+            "duration",
             "expire",
+            "growing",
             "dirty"
         ):
-            setattr(self, name, state[name])
+            value = state.get(name, None)
+            setattr(self, name, value)
         self.transient = dict()
 
     @classmethod
@@ -189,8 +202,18 @@ class Session(object):
     def flush(self, request = None):
         self.mark(dirty = False)
 
-    def mark(self, dirty = True):
+    def mark(self, dirty = True, extend = False):
         self.dirty = dirty
+        if extend: self.extend()
+
+    def extend(self, duration = None):
+        duration = duration or self.duration
+        if not duration: return
+        self.modify = time.time()
+        self.duration = duration
+        self.expire = self.modify + duration
+        self.mark()
+        return self.expire
 
     def is_expired(self):
         has_expire = hasattr(self, "expire")
@@ -294,11 +317,15 @@ class DataSession(Session):
         except KeyError: return Session.__getitem__(self, key)
 
     def __setitem__(self, key, value):
-        self.mark(); return self.data.__setitem__(key, value)
+        self.mark(extend = self.growing)
+        return self.data.__setitem__(key, value)
 
     def __delitem__(self, key):
-        try: self.mark(); return self.data.__delitem__(key)
-        except KeyError: return Session.__delitem__(self, key)
+        try:
+            self.mark(extend = self.growing)
+            return self.data.__delitem__(key)
+        except KeyError:
+            return Session.__delitem__(self, key)
 
     def __iter__(self):
         return self._merged.__iter__()
@@ -442,6 +469,7 @@ class FileSession(DataSession):
 
     @classmethod
     def close(cls):
+        if not cls.SHELVE: return
         cls.SHELVE.close()
         cls.SHELVE = None
 
