@@ -75,13 +75,13 @@ class SessionTest(unittest.TestCase):
         self.assertEqual(session["first"], 1)
         self.assertEqual(session["second"], 2)
 
-        appier.FileSession.close()
-
         del session["first"]
 
         self.assertRaises(KeyError, lambda: session["first"])
         self.assertEqual(session.get("first"), None)
 
+        appier.FileSession.close()
+
     def test_expire(self):
         expire = datetime.timedelta(days = 0)
         session = appier.MemorySession.new(expire = expire)
@@ -95,10 +95,17 @@ class SessionTest(unittest.TestCase):
         session = appier.MemorySession.new(expire = expire)
 
         self.assertEqual(session.is_expired(), False)
+        self.assertEqual(session.duration, 86400)
 
         session = appier.MemorySession.get_s(session.sid)
         self.assertNotEqual(session, None)
 
+        expire = session.extend(duration = 60)
+
+        self.assertEqual(session.duration, 60)
+        self.assertEqual(session.expire, expire)
+        self.assertEqual(session.expire, session.modify + 60)
+
     def test_transient(self):
         session = appier.MemorySession.new()
 
