@@ -43,6 +43,7 @@ import hashlib
 import tempfile
 
 from . import util
+from . import crypt
 from . import legacy
 from . import common
 
@@ -57,8 +58,8 @@ class Type(object):
 class File(Type):
 
     def __init__(self, file):
-        file_t = type(file)
-        if file_t == dict: self.build_b64(file)
+        if isinstance(file, legacy.BYTES): self.build_d(file)
+        elif isinstance(file, dict): self.build_b64(file)
         elif isinstance(file, tuple): self.build_t(file)
         elif isinstance(file, File): self.build_i(file)
         else: self.build_f(file)
@@ -72,19 +73,28 @@ class File(Type):
     def __len__(self):
         return self.size
 
+    def build_d(self, file_d, name = "default"):
+        self.build_t((name, None, file_d))
+
     def build_b64(self, file_m):
         name = file_m["name"]
         data_b64 = file_m["data"]
+        hash = file_m.get("hash", None)
         mime = file_m.get("mime", None)
+        etag = file_m.get("etag", None)
 
         is_valid = name and data_b64
-        data = base64.b64decode(data_b64) if is_valid else None
+        data_b64_b = legacy.bytes(data_b64)
+        data = base64.b64decode(data_b64_b) if is_valid else None
+        data_b64 = legacy.str(data_b64)
         size = len(data) if is_valid else 0
-        etag = self._etag(data)
+        hash = hash or self._hash(data)
+        etag = etag or self._etag(data)
 
         self.data = data
         self.data_b64 = data_b64
         self.file = None
+        self.hash = hash
         self.size = size
         self.file_name = name
         self.mime = mime
@@ -95,34 +105,39 @@ class File(Type):
 
         is_valid = name and data
         data_b64 = base64.b64encode(data) if is_valid else None
+        data_b64 = legacy.str(data_b64)
         size = len(data) if is_valid else 0
         etag = self._etag(data)
+        hash = self._hash(data)
 
         self.data = data
         self.data_b64 = data_b64
         self.file = None
+        self.hash = hash
         self.size = size
         self.file_name = name
         self.mime = content_type
         self.etag = etag
 
     def build_i(self, file):
+        self.data = file.data
+        self.data_b64 = file.data_b64
         self.file = file.file
+        self.hash = file.hash
         self.size = file.size
         self.file_name = file.file_name
         self.mime = file.mime
         self.etag = file.etag
-        self.data = file.data
-        self.data_b64 = file.data_b64
 
     def build_f(self, file):
+        self.data = None
+        self.data_b64 = None
         self.file = file
+        self.hash = hash
         self.size = file.content_length
         self.file_name = file.filename
         self.mime = file.content_type
         self.etag = None
-        self.data = None
-        self.data_b64 = None
 
         self._flush()
 
@@ -133,7 +148,9 @@ class File(Type):
         return dict(
             name = self.file_name,
             data = self.data_b64,
-            mime = self.mime
+            hash = self.hash,
+            mime = self.mime,
+            etag = self.etag
         ) if self.is_valid() else None
 
     def is_valid(self):
@@ -142,6 +159,12 @@ class File(Type):
     def is_empty(self):
         return self.size <= 0
 
+    def _hash(self, data):
+        if not data: return None
+        hash = hashlib.sha256(data)
+        digest = hash.hexdigest()
+        return digest
+
     def _etag(self, data):
         if not data: return None
         hash = hashlib.md5(data)
@@ -162,6 +185,8 @@ class File(Type):
 
         self.data = data
         self.data_b64 = base64.b64encode(data)
+        self.data_b64 = legacy.str(self.data_b64)
+        self.hash = self._hash(data)
         self.size = len(data)
         self.etag = self._etag(data)
 
@@ -199,7 +224,7 @@ class Files(Type):
             self._files.append(_file)
 
     def json_v(self, *args, **kwargs):
-        return [file.json_v() for file in self._files]
+        return [file.json_v(*args, **kwargs) for file in self._files]
 
     def is_empty(self):
         return len(self._files) == 0
@@ -213,29 +238,48 @@ class ImageFile(File):
         File.build_b64(self, file_m)
         self.width = file_m.get("width", 0)
         self.height = file_m.get("height", 0)
+        self.format = file_m.get("format", None)
+        self._ensure_all()
 
     def build_t(self, file_t):
         File.build_t(self, file_t)
-        self.width, self.height = self._size()
+        self._ensure_all()
 
     def build_i(self, file):
         File.build_i(self, file)
-        self.width = file.width
-        self.height = file.height
+        self.width = file.width if hasattr(file, "width") else 0
+        self.height = file.height if hasattr(file, "height") else 0
+        self.format = file.height if hasattr(file, "format") else None
+        self._ensure_all()
 
     def build_f(self, file):
         File.build_f(self, file)
-        self.width, self.height = self._size()
+        self._ensure_all()
 
     def json_v(self, *args, **kwargs):
         return dict(
             name = self.file_name,
             data = self.data_b64,
+            hash = self.hash,
             mime = self.mime,
+            etag = self.etag,
             width = self.width,
-            height = self.height
+            height = self.height,
+            format = self.format
         ) if self.is_valid() else None
 
+    def _ensure_all(self):
+        self._ensure_size()
+        self._ensure_mime()
+
+    def _ensure_size(self):
+        if self.width and self.height: return
+        self.width, self.height = self._size()
+
+    def _ensure_mime(self):
+        if self.format and self.mime: return
+        self.format, self.mime = self._mime()
+
     def _size(self):
         try: return self._size_image()
         except: return self._size_default()
@@ -254,6 +298,25 @@ class ImageFile(File):
     def _size_default(self):
         return (0, 0)
 
+    def _mime(self):
+        try: return self._mime_image()
+        except: return self._mime_default()
+
+    def _mime_image(self):
+        import PIL.Image
+        if not self.data: return self._size_default()
+        buffer = legacy.BytesIO(self.data)
+        try:
+            image = PIL.Image.open(buffer)
+            format = image.format.lower()
+            mime = PIL.Image.MIME[image.format]
+        finally:
+            buffer.close()
+        return format, mime
+
+    def _mime_default(self):
+        return None, "application/octet-stream"
+
 class ImageFiles(Files):
 
     def base(self):
@@ -263,15 +326,47 @@ def image(width = None, height = None, format = "png"):
 
     class _ImageFile(ImageFile):
 
+        def build_b64(self, file_m):
+            ImageFile.build_b64(self, file_m)
+
+            need_resize = self.need_resize(
+                width_o = self.width,
+                height_o = self.height,
+                format_o = self.format
+            )
+            if not need_resize: return
+
+            data_b64 = legacy.bytes(self.data_b64)
+            data = base64.b64decode(data_b64)
+            data = self.resize(data)
+            data_b64 = base64.b64encode(data)
+            data_b64 = legacy.str(data_b64)
+
+            # updates the data attribute of the file map wit the
+            # "newly" resize data value coming from the resize
+            # operation, note that this "new" data may have different
+            # width, height and format from the original one
+            file_m["data"] = data_b64
+
+            # removes a series of keys from the file map as
+            # they are no longer reliable and may reflect
+            # different values from the ones contained in data
+            # theses should be update b y the "ensure" methods
+            for name in ("width", "height", "format"):
+                if name in file_m: del file_m[name]
+
+            ImageFile.build_b64(self, file_m)
+
         def build_t(self, file_t):
             name, content_type, data = file_t
-            try: _data = self._resize(data)
+            try: _data = self.resize(data)
             except: _data = data
             file_t = (name, content_type, _data)
             ImageFile.build_t(self, file_t)
 
-        def _resize(self, data):
+        def resize(self, data = None):
             import PIL.Image
+            data = data or self.data
             if not data: return data
 
             is_resized = True if width or height else False
@@ -282,7 +377,7 @@ def image(width = None, height = None, format = "png"):
             out_buffer = legacy.BytesIO()
             try:
                 image = PIL.Image.open(in_buffer)
-                image = self.__resize(image, size)
+                image = self._resize(image, size)
                 image.save(out_buffer, format)
                 data = out_buffer.getvalue()
             finally:
@@ -291,7 +386,18 @@ def image(width = None, height = None, format = "png"):
 
             return data
 
-        def __resize(self, image, size):
+        def need_resize(
+            self,
+            width_o = None,
+            height_o = None,
+            format_o = None
+        ):
+            if width and not width == width_o: return True
+            if height and not height == height_o: return True
+            if format and not format == format_o: return True
+            return False
+
+        def _resize(self, image, size):
             import PIL.Image
 
             # unpacks the provided tuple containing the size dimension into the
@@ -360,10 +466,11 @@ def images(width = None, height = None, format = "png"):
 class Reference(Type):
     pass
 
-def reference(target, name = None, eager = False):
+def reference(target, name = None, dumpall = False):
     name = name or "id"
     target_t = type(target)
     is_reference = target_t in legacy.STRINGS
+    reserved = ("id", "_target", "_object", "_type", "__dict__")
 
     class _Reference(Reference):
 
@@ -390,6 +497,14 @@ def reference(target, name = None, eager = False):
             if has_unicode: return self._object.__unicode__()
             else: return legacy.UNICODE(self._object) or legacy.UNICODE()
 
+        def __eq__(self, other):
+            if other == None: return self.id in ("", b"", None)
+            if isinstance(other, Reference): return self.equals(other)
+            return id(self) == id(other)
+
+        def __ne__(self, other):
+            return not self.__eq__(other)
+
         def __len__(self):
             is_empty = self.id in ("", b"", None)
             return 0 if is_empty else 1
@@ -405,6 +520,11 @@ def reference(target, name = None, eager = False):
             raise AttributeError("'%s' not found" % name)
 
         def __setattr__(self, name, value):
+            # in case the name that is being set is not part of the reserved
+            # names for the reference underlying structure the object resolution
+            # is triggered to make sure the underlying object exists and is loaded
+            if not name in reserved: self.resolve()
+
             # verifies if the reference object exists in the current
             # reference instance, that's the case if the object name is
             # defined in the dictionary and the referenced object contains
@@ -439,7 +559,10 @@ def reference(target, name = None, eager = False):
             meta = getattr(_target, name)
             return meta.get("type", legacy.UNICODE)
 
-        def build(self, id):
+        def build(self, id, cast = True):
+            is_unset = id in ("", b"", None)
+            cast = cast and not is_unset
+            if cast: id = self._target.cast(name, id)
             self.id = id
             self._object = None
 
@@ -452,23 +575,25 @@ def reference(target, name = None, eager = False):
             self._object = object
 
         def ref_v(self, *args, **kwargs):
-            return self.value()
+            return self.val()
 
         def json_v(self, *args, **kwargs):
-            if eager: self.resolve(); return self._object
-            else: return self.value()
+            if dumpall: return self.resolve()
+            return self.val()
 
         def map_v(self, *args, **kwargs):
-            value = self.resolve()
-            if not value: return value
+            resolve = kwargs.get("resolve", True)
+            value = self.resolve() if resolve else self._object
+            if resolve and not value: return value
+            if not value: return self.val()
             return value.map(*args, **kwargs)
 
-        def value(self):
+        def val(self):
             is_empty = self.id in ("", b"", None)
             if is_empty: return None
             return self._type(self.id)
 
-        def resolve(self, strict = False):
+        def resolve(self, *args, **kwargs):
             # verifies if the underlying object reference exists
             # in the current names dictionary and if it exists
             # verifies if it's valid (value is valid) if that's
@@ -489,10 +614,11 @@ def reference(target, name = None, eager = False):
             # to be used in the resolution of the reference and uses the
             # data source based get attribute to retrieve the object
             # that represents the reference
-            kwargs = {
-                name : self._target.cast(name, self.id)
-            }
-            _object = self._target.get(raise_e = strict, **kwargs)
+            kwargs = dict(kwargs)
+            kwargs[name] = self._target.cast(name, self.id)
+            kwargs["raise_e"] = kwargs.get("raise_e", False)
+            kwargs["eager_l"] = kwargs.get("eager_l", False)
+            _object = self._target.get(*args, **kwargs)
 
             # sets the resolved object (using the current id attribute)
             # in the current instance's dictionary and then returns this
@@ -500,14 +626,24 @@ def reference(target, name = None, eager = False):
             self.__dict__["_object"] = _object
             return _object
 
+        def equals(self, other):
+            if not self.__class__ == other.__class__: return False
+            if not self._target == other._target: return False
+            if not self.id == other.id: return False
+            return True
+
+        def is_resolved(self):
+            exists = "_object" in self.__dict__
+            return True if exists and self._object else False
+
     return _Reference
 
 class References(Type):
     pass
 
-def references(target, name = None, eager = False):
+def references(target, name = None, dumpall = False):
     name = name or "id"
-    reference_c = reference(target, name = name, eager = eager)
+    reference_c = reference(target, name = name, dumpall = dumpall)
 
     class _References(References):
 
@@ -565,11 +701,14 @@ def references(target, name = None, eager = False):
 
         def set_ids(self, ids):
             ids = ids or []
+            self.ids = []
             for id in ids:
                 if id == "" or id == None: continue
                 object = reference_c(id)
+                object_id = object.id
+                self.ids.append(object_id)
                 self.objects.append(object)
-                self.objects_m[id] = object
+                self.objects_m[object_id] = object
 
         def ref_v(self, *args, **kwargs):
             return [object.ref_v(*args, **kwargs) for object in self.objects]
@@ -581,10 +720,10 @@ def references(target, name = None, eager = False):
             return [object.map_v(*args, **kwargs) for object in self.objects]
 
         def list(self):
-            return [object.value() for object in self.objects]
+            return [object.val() for object in self.objects]
 
-        def resolve(self):
-            return [object.resolve() for object in self.objects]
+        def resolve(self, *args, **kwargs):
+            return [object.resolve(*args, **kwargs) for object in self.objects]
 
         def is_empty(self):
             ids_l = len(self.ids)
@@ -611,4 +750,86 @@ def references(target, name = None, eager = False):
             if is_object: id = getattr(id, self._name)
             return id in self.objects_m
 
+        def is_resolved(self):
+            if not self.objects: return True
+            return self.objects[0].is_resolved()
+
     return _References
+
+class Encrypted(Type):
+
+    PADDING = ":encrypted"
+
+def encrypted(cipher = "spritz", key = None, encoding = "utf-8"):
+    key = key or None
+
+    class _Encrypted(Encrypted):
+
+        def __init__(self, value):
+            cls = self.__class__
+            util.verify(
+                isinstance(value, legacy.ALL_STRINGS) or\
+                isinstance(value, Encrypted)
+            )
+            self.key = key or common.base().APP.crypt_secret
+            self.key = legacy.bytes(self.key)
+            if isinstance(value, Encrypted): self.build_i(value)
+            elif value.endswith(cls.PADDING): self.build_e(value)
+            else: self.build(value)
+
+        def __str__(self):
+            return self.value
+
+        def __unicode__(self):
+            return self.value
+
+        def __len__(self):
+            return self.value.__len__()
+
+        def __iter__(self):
+            return self.value.__iter__()
+
+        def __bool__(self):
+            return bool(self.value)
+
+        def build(self, value):
+            self.value = value
+            self.encrypted = self._encrypt(value)
+
+        def build_e(self, encrypted):
+            self.encrypted = encrypted
+            self.value = self._decrypt(encrypted)
+
+        def build_i(self, instance):
+            self.key = instance.key
+            self.value = instance.value
+            self.encrypted = instance.encrypted
+
+        def json_v(self, *args, **kwargs):
+            return self.encrypted
+
+        def _encrypt(self, value, strict = False):
+            if not self.key and not strict: return value
+            cls = self.__class__
+            value = legacy.bytes(value, encoding = encoding)
+            cipher_i = crypt.Cipher.new(cipher, self.key)
+            encrypted = cipher_i.encrypt(value)
+            encrypted = base64.b64encode(encrypted)
+            encrypted = legacy.str(encrypted, encoding = encoding)
+            return encrypted + cls.PADDING
+
+        def _decrypt(self, value, strict = False):
+            if not self.key and not strict: return value
+            cls = self.__class__
+            util.verify(value.endswith(cls.PADDING))
+            value = value[:-len(cls.PADDING)]
+            value = legacy.bytes(value)
+            value = base64.b64decode(value)
+            cipher_i = crypt.Cipher.new(cipher, self.key)
+            decrypted = cipher_i.decrypt(value)
+            decrypted = legacy.str(decrypted, encoding = encoding)
+            return decrypted
+
+    return _Encrypted
+
+secure = encrypted()
