@@ -46,8 +46,11 @@ import uuid
 import types
 import locale
 import hashlib
-import threading
+import calendar
+import datetime
 import functools
+import threading
+import mimetypes
 import contextlib
 import subprocess
 
@@ -102,6 +105,7 @@ ALIAS = {
     "filters[]" : "find_d",
     "filter_def" : "find_d",
     "filter_string" : "find_s",
+    "filter_name" : "find_n",
     "insensitive" : "find_i",
     "order" : "sort",
     "offset" : "skip",
@@ -118,6 +122,7 @@ FIND_TYPES = dict(
     find_d = to_find,
     find_i = bool,
     find_t = legacy.UNICODE,
+    find_n = legacy.UNICODE,
     sort = to_sort,
     meta = bool,
     fields = list
@@ -226,6 +231,13 @@ def email_base(base):
     _name, email = email_parts(base)
     return email
 
+def date_to_timestamp(value, format = "%d/%m/%Y"):
+    if not value: return None
+    try: value = datetime.datetime.strptime(value, format)
+    except: return None
+    value = value.utctimetuple()
+    return calendar.timegm(value)
+
 def obfuscate(value, display_l = 3, token = "*"):
     value_l = len(value)
     display_l = min([value_l, display_l])
@@ -740,7 +752,7 @@ def parse_multipart(data, boundary):
     files = dict()
 
     boundary = boundary.strip()
-    boundary_base = "--" + boundary[9:]
+    boundary_base = "--" + boundary[9:].strip("\"")
     boundary_value = legacy.bytes(boundary_base + "\r\n")
     boundary_extra = legacy.bytes(boundary_base + "--" + "\r\n")
     boundary_extra_l = len(boundary_extra)
@@ -800,8 +812,8 @@ def parse_multipart(data, boundary):
         # values are going to be used to decide on whether the current
         # part is a file or a normal key value attribute
         content_type = headers.get("content-type", None)
-        name = parts.get("name", b"\"undefined\"")[1:-1]
-        filename = parts.get("filename", b"")[1:-1]
+        name = parts.get("name", b"\"undefined\"").strip(b"\"")
+        filename = parts.get("filename", b"").strip(b"\"")
 
         # decodes the various content disposition values into an unicode
         # based string so that may be latter be used safely inside the
@@ -960,6 +972,27 @@ def ensure_login(self, function, token = None, request = None):
         token = token
     )
 
+def dict_merge(first, second, override = True):
+    if not override: first, second = second, first
+    final = dict(first)
+    final.update(second)
+    return final
+
+def cached(function):
+
+    name = function.__name__
+
+    @functools.wraps(function)
+    def _cached(self, *args, **kwargs):
+        properties = self.request.properties
+        exists = name in properties
+        if exists: return properties[name]
+        value = function(self, *args, **kwargs)
+        properties[name] = value
+        return value
+
+    return _cached
+
 def private(function):
 
     @functools.wraps(function)
@@ -1024,7 +1057,7 @@ def route(url, method = "GET", async = False, json = False):
 
     return decorator
 
-def error_handler(code, json = False):
+def error_handler(code, scope = None, json = False):
 
     def decorator(function, *args, **kwargs):
         if is_detached(function): delay(function, *args, **kwargs)
@@ -1033,7 +1066,7 @@ def error_handler(code, json = False):
 
     def delay(function, *args, **kwargs):
         global CREATION_COUNTER
-        error = (code, json)
+        error = (code, scope, json)
         if not hasattr(function, "_errors"): function._errors = []
         function._errors.append(error)
         function.creation_counter = CREATION_COUNTER
@@ -1042,7 +1075,7 @@ def error_handler(code, json = False):
 
     return decorator
 
-def exception_handler(exception, json = False):
+def exception_handler(exception, scope = None, json = False):
 
     def decorator(function, *args, **kwargs):
         if is_detached(function): delay(function, *args, **kwargs)
@@ -1051,7 +1084,7 @@ def exception_handler(exception, json = False):
 
     def delay(function, *args, **kwargs):
         global CREATION_COUNTER
-        _exception = (exception, json)
+        _exception = (exception, scope, json)
         if not hasattr(function, "_exceptions"): function._exceptions = []
         function._exceptions.append(_exception)
         function.creation_counter = CREATION_COUNTER
@@ -1187,6 +1220,26 @@ class FileTuple(tuple):
     typical python class interface.
     """
 
+    @classmethod
+    def from_file(cls, file, name = None, mime = None):
+        data = file.read()
+        file_tuple = cls((name, mime, data))
+        return file_tuple
+
+    @classmethod
+    def from_path(cls, path, name = None, mime = None, guess = True):
+        mime = cls.guess(path) if mime == None and guess else mime
+        file = open(path, "rb")
+        try: file_tuple = cls.from_file(file, name = name, mime = mime)
+        finally: file.close()
+        return file_tuple
+
+    @classmethod
+    def guess(self, name):
+        mime = mimetypes.guess_type(name, strict = False)[0]
+        if mime: return mime
+        return None
+
     def read(self, count = None):
         contents = self[2]
         return contents
@@ -1224,3 +1277,14 @@ class BaseThread(threading.Thread):
         if not self.owner: return
         self.owner.start()
         self.owner = None
+
+class JSONEncoder(json.JSONEncoder):
+
+    def __init__(self, *args, **kwargs):
+        self.permissive = kwargs.pop("permissive", True)
+        json.JSONEncoder.__init__(self, *args, **kwargs)
+
+    def default(self, obj, **kwargs):
+        if hasattr(obj, "json_v"): return obj.json_v()
+        if self.permissive: return str(obj)
+        return json.JSONEncoder.default(self, obj, **kwargs)
