@@ -80,7 +80,7 @@ def validate(method = None, methods = [], object = None, ctx = None, build = Tru
     # uses the provided method to retrieves the complete
     # set of methods to be used for validation, this provides
     # an extra level of indirection
-    methods = method and method() or methods
+    methods = method() if method else methods
     errors = []
 
     # verifies if the provided object is valid in such case creates
@@ -126,221 +126,269 @@ def validate(method = None, methods = [], object = None, ctx = None, build = Tru
     # model name with the sequence of errors and the validated object (state)
     return errors_map, object
 
-def validate_b(method = None, methods = [], object = None, build = True):
+def validate_b(method = None, methods = [], object = None, ctx = None, build = True):
     errors_map, object = validate(
         method = method,
         methods = methods,
         object = object,
+        ctx = ctx,
         build = build
     )
     result = False if errors_map else True
     return result
 
+def validate_e(method = None, methods = [], object = None, ctx = None, build = True):
+    errors_map, object = validate(
+        method = method,
+        methods = methods,
+        object = object,
+        ctx = ctx,
+        build = build
+    )
+    if not errors_map: return
+    raise exceptions.ValidationError(errors_map, object)
+
 def safe(comparision):
     try: return comparision()
     except TypeError: return False
 
-def eq(name, value_c, message = "must be equal to %s"):
+def eq(name, value_c, message = "must be equal to %s", locale = True):
     def validation(object, ctx):
         value = object.get(name, None)
         if value == None: return True
         if value == value_c: return True
-        raise exceptions.ValidationInternalError(name, message % str(value_c))
+        message_l = _to_locale(message) if locale else message
+        raise exceptions.ValidationInternalError(name, message_l % str(value_c))
     return validation
 
-def gt(name, value_c, message = "must be greater than %s"):
+def gt(name, value_c, message = "must be greater than %s", locale = True):
     def validation(object, ctx):
         value = object.get(name, None)
         if value == None: return True
         if safe(lambda: value > value_c): return True
-        raise exceptions.ValidationInternalError(name, message % str(value_c))
+        message_l = _to_locale(message) if locale else message
+        raise exceptions.ValidationInternalError(name, message_l % str(value_c))
     return validation
 
-def gte(name, value_c, message = "must be greater than or equal to %s"):
+def gte(name, value_c, message = "must be greater than or equal to %s", locale = True):
     def validation(object, ctx):
         value = object.get(name, None)
         if value == None: return True
         if safe(lambda: value >= value_c): return True
-        raise exceptions.ValidationInternalError(name, message % str(value_c))
+        message_l = _to_locale(message) if locale else message
+        raise exceptions.ValidationInternalError(name, message_l % str(value_c))
     return validation
 
-def lt(name, value_c, message = "must be less than %s"):
+def lt(name, value_c, message = "must be less than %s", locale = True):
     def validation(object, ctx):
         value = object.get(name, None)
         if value == None: return True
         if safe(lambda: value < value_c): return True
-        raise exceptions.ValidationInternalError(name, message % str(value_c))
+        message_l = _to_locale(message) if locale else message
+        raise exceptions.ValidationInternalError(name, message_l % str(value_c))
     return validation
 
-def lte(name, value_c, message = "must be less than or equal to %s"):
+def lte(name, value_c, message = "must be less than or equal to %s", locale = True):
     def validation(object, ctx):
         value = object.get(name, None)
         if value == None: return True
         if safe(lambda: value <= value_c): return True
-        raise exceptions.ValidationInternalError(name, message % str(value_c))
+        message_l = _to_locale(message) if locale else message
+        raise exceptions.ValidationInternalError(name, message_l % str(value_c))
     return validation
 
-def not_null(name, message = "value is not set"):
+def not_null(name, message = "value is not set", locale = True):
     def validation(object, ctx):
         value = object.get(name, None)
         if not value == None: return True
-        raise exceptions.ValidationInternalError(name, message)
+        message_l = _to_locale(message) if locale else message
+        raise exceptions.ValidationInternalError(name, message_l)
     return validation
 
-def not_empty(name, message = "value is empty"):
+def not_empty(name, message = "value is empty", locale = True):
     def validation(object, ctx):
         value = object.get(name, None)
         if value == None: return True
         if len(value): return True
-        raise exceptions.ValidationInternalError(name, message)
+        message_l = _to_locale(message) if locale else message
+        raise exceptions.ValidationInternalError(name, message_l)
     return validation
 
-def not_false(name, message = "value is false"):
+def not_false(name, message = "value is false", locale = True):
     def validation(object, ctx):
         value = object.get(name, None)
         if value == None: return True
         if not value == False: return True
-        raise exceptions.ValidationInternalError(name, message)
+        message_l = _to_locale(message) if locale else message
+        raise exceptions.ValidationInternalError(name, message_l)
     return validation
 
-def is_in(name, values, message = "value is not in set"):
+def is_in(name, values, message = "value is not in set", locale = True):
     def validation(object, ctx):
         value = object.get(name, None)
         if value == None: return True
         if value in values: return True
-        raise exceptions.ValidationInternalError(name, message)
+        message_l = _to_locale(message) if locale else message
+        raise exceptions.ValidationInternalError(name, message_l)
     return validation
 
-def is_simple(name, message = "value contains invalid characters"):
+def is_simple(name, message = "value contains invalid characters", locale = True):
     def validation(object, ctx):
         value = object.get(name, None)
         if value == None: return True
         if value == "": return True
         if SIMPLE_REGEX.match(value): return True
-        raise exceptions.ValidationInternalError(name, message)
+        message_l = _to_locale(message) if locale else message
+        raise exceptions.ValidationInternalError(name, message_l)
     return validation
 
-def is_email(name, message = "value is not a valid email"):
+def is_lower(name, message = "value contains upper cased characters", locale = True):
+    def validation(object, ctx):
+        value = object.get(name, None)
+        if value == None: return True
+        if value == "": return True
+        if value == value.lower(): return True
+        message_l = _to_locale(message) if locale else message
+        raise exceptions.ValidationInternalError(name, message_l)
+    return validation
+
+def is_email(name, message = "value is not a valid email", locale = True):
     def validation(object, ctx):
         value = object.get(name, None)
         if value == None: return True
         if value == "": return True
         if EMAIL_REGEX.match(value): return True
-        raise exceptions.ValidationInternalError(name, message)
+        message_l = _to_locale(message) if locale else message
+        raise exceptions.ValidationInternalError(name, message_l)
     return validation
 
-def is_url(name, message = "value is not a valid url"):
+def is_url(name, message = "value is not a valid url", locale = True):
     def validation(object, ctx):
         value = object.get(name, None)
         if value == None: return True
         if value == "": return True
         if URL_REGEX.match(value): return True
-        raise exceptions.ValidationInternalError(name, message)
+        message_l = _to_locale(message) if locale else message
+        raise exceptions.ValidationInternalError(name, message_l)
     return validation
 
-def is_regex(name, regex, message = "value has incorrect format"):
+def is_regex(name, regex, message = "value has incorrect format", locale = True):
     def validation(object, ctx):
         value = object.get(name, None)
         if value == None: return True
         if value == "": return True
         match = re.match(regex, value)
         if match: return True
-        raise exceptions.ValidationInternalError(name, message)
+        message_l = _to_locale(message) if locale else message
+        raise exceptions.ValidationInternalError(name, message_l)
     return validation
 
-def field_eq(name, field, message = "must be equal to %s"):
+def field_eq(name, field, message = "must be equal to %s", locale = True):
     def validation(object, ctx):
         name_v = object.get(name, None)
         field_v = object.get(field, None)
         if name_v == None: return True
         if field_v == None: return True
         if name_v == field_v: return True
-        raise exceptions.ValidationInternalError(name, message % field)
+        message_l = _to_locale(message) if locale else message
+        raise exceptions.ValidationInternalError(name, message_l % field)
     return validation
 
-def field_gt(name, field, message = "must be greater than %s"):
+def field_gt(name, field, message = "must be greater than %s", locale = True):
     def validation(object, ctx):
         name_v = object.get(name, None)
         field_v = object.get(field, None)
         if name_v == None: return True
         if field_v == None: return True
         if safe(lambda: name_v > field_v): return True
-        raise exceptions.ValidationInternalError(name, message % field)
+        message_l = _to_locale(message) if locale else message
+        raise exceptions.ValidationInternalError(name, message_l % field)
     return validation
 
-def field_gte(name, field, message = "must be greater or equal than %s"):
+def field_gte(name, field, message = "must be greater or equal than %s", locale = True):
     def validation(object, ctx):
         name_v = object.get(name, None)
         field_v = object.get(field, None)
         if name_v == None: return True
         if field_v == None: return True
         if safe(lambda: name_v >= field_v): return True
-        raise exceptions.ValidationInternalError(name, message % field)
+        message_l = _to_locale(message) if locale else message
+        raise exceptions.ValidationInternalError(name, message_l % field)
     return validation
 
-def field_lt(name, field, message = "must be less than %s"):
+def field_lt(name, field, message = "must be less than %s", locale = True):
     def validation(object, ctx):
         name_v = object.get(name, None)
         field_v = object.get(field, None)
         if name_v == None: return True
         if field_v == None: return True
         if safe(lambda: name_v < field_v): return True
-        raise exceptions.ValidationInternalError(name, message % field)
+        message_l = _to_locale(message) if locale else message
+        raise exceptions.ValidationInternalError(name, message_l % field)
     return validation
 
-def field_lte(name, field, message = "must be less or equal than %s"):
+def field_lte(name, field, message = "must be less or equal than %s", locale = True):
     def validation(object, ctx):
         name_v = object.get(name, None)
         field_v = object.get(field, None)
         if name_v == None: return True
         if field_v == None: return True
         if safe(lambda: name_v <= field_v): return True
-        raise exceptions.ValidationInternalError(name, message % field)
+        message_l = _to_locale(message) if locale else message
+        raise exceptions.ValidationInternalError(name, message_l % field)
     return validation
 
-def string_gt(name, size, message = "must be larger than %d characters"):
+def string_gt(name, size, message = "must be larger than %d characters", locale = True):
     def validation(object, ctx):
         value = object.get(name, None)
         if value == None: return True
+        if value == "": return True
         if len(value) > size: return True
-        raise exceptions.ValidationInternalError(name, message % size)
+        message_l = _to_locale(message) if locale else message
+        raise exceptions.ValidationInternalError(name, message_l % size)
     return validation
 
-def string_lt(name, size, message = "must be smaller than %d characters"):
+def string_lt(name, size, message = "must be smaller than %d characters", locale = True):
     def validation(object, ctx):
         value = object.get(name, None)
         if value == None: return True
+        if value == "": return True
         if len(value) < size: return True
-        raise exceptions.ValidationInternalError(name, message % size)
+        message_l = _to_locale(message) if locale else message
+        raise exceptions.ValidationInternalError(name, message_l % size)
     return validation
 
-def string_eq(name, size, message = "must be exactly %d characters"):
+def string_eq(name, size, message = "must be exactly %d characters", locale = True):
     def validation(object, ctx):
         value = object.get(name, None)
         if value == None: return True
+        if value == "": return True
         if len(value) == size: return True
-        raise exceptions.ValidationInternalError(name, message % size)
+        message_l = _to_locale(message) if locale else message
+        raise exceptions.ValidationInternalError(name, message_l % size)
     return validation
 
-def equals(first_name, second_name, message = "value is not equal to %s"):
+def equals(first_name, second_name, message = "value is not equal to %s", locale = True):
     def validation(object, ctx):
         first_value = object.get(first_name, None)
         second_value = object.get(second_name, None)
         if first_value == None: return True
         if second_value == None: return True
         if first_value == second_value: return True
-        raise exceptions.ValidationInternalError(first_name, message % second_name)
+        message_l = _to_locale(message) if locale else message
+        raise exceptions.ValidationInternalError(first_name, message_l % second_name)
     return validation
 
-def not_past(name, message = "date is in the past"):
+def not_past(name, message = "date is in the past", locale = True):
     def validation(object, ctx):
         value = object.get(name, None)
         if value == None: return True
         if value >= datetime.datetime.utcnow(): return True
-        raise exceptions.ValidationInternalError(name, message)
+        message_l = _to_locale(message) if locale else message
+        raise exceptions.ValidationInternalError(name, message_l)
     return validation
 
-def not_duplicate(name, collection, message = "value is duplicate"):
+def not_duplicate(name, collection, message = "value is duplicate", locale = True):
     def validation(object, ctx):
         _id = object.get("_id", None)
         value = object.get(name, None)
@@ -351,10 +399,11 @@ def not_duplicate(name, collection, message = "value is duplicate"):
         item = _collection.find_one({name : value})
         if not item: return True
         if str(item["_id"]) == str(_id): return True
-        raise exceptions.ValidationInternalError(name, message)
+        message_l = _to_locale(message) if locale else message
+        raise exceptions.ValidationInternalError(name, message_l)
     return validation
 
-def all_different(name, name_ref = None, message = "has duplicates"):
+def all_different(name, name_ref = None, message = "has duplicates", locale = True):
     def validation(object, ctx):
         # uses the currently provided context to retrieve
         # the definition of the name to be validation and
@@ -390,10 +439,11 @@ def all_different(name, name_ref = None, message = "has duplicates"):
         # same the sequence is considered to not contain duplicates
         values_set = set(values)
         if len(value) == len(values_set): return True
-        raise exceptions.ValidationInternalError(name, message)
+        message_l = _to_locale(message) if locale else message
+        raise exceptions.ValidationInternalError(name, message_l)
     return validation
 
-def no_self(name, name_ref = None, message = "contains self"):
+def no_self(name, name_ref = None, message = "contains self", locale = True):
     def validation(object, ctx):
         # uses the currently provided context to retrieve
         # the definition of the name to be validation and
@@ -430,5 +480,9 @@ def no_self(name, name_ref = None, message = "contains self"):
         # exception indicating the validation problem
         exists = _id in values
         if not exists: return True
-        raise exceptions.ValidationInternalError(name, message)
+        message_l = _to_locale(message) if locale else message
+        raise exceptions.ValidationInternalError(name, message_l)
     return validation
+
+def _to_locale(value):
+    return common.base().to_locale(value)
